library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity llc_iface is
port
(
	tm4_glbclk0	: in std_logic;
	reset_n		: in std_logic;

	sigin_wanted	: out std_logic;
	signals_out	: out std_logic_vector(31 downto 0);
	signals_in	: in std_logic_vector(31 downto 0);
	sigin_ready	: in std_logic;

	tm4_fir_bclk	: out std_logic;
	tm4_fir_resetn	: out std_logic;
	tm4_fir_ma	: out std_logic_vector(0 to 6);
	tm4_fir_md	: inout std_logic_vector(0 to 15);
	tm4_fir_mwrn	: out std_logic;
	tm4_fir_mcsn	: out std_logic;
	tm4_fir_mcan	: in std_logic;
	tm4_fir_tean	: in std_logic;

	tm4_fir_coldfire : out std_logic;
	tm4_fir_lendian	: out std_logic;
	tm4_fir_m8bit	: out std_logic;
	tm4_fir_mcmode	: out std_logic
);
end llc_iface;

architecture rtl of llc_iface is
	component llc_pll
	port
	(
		inclk0	: in std_logic;
		c0	: out std_logic;
		c1	: out std_logic
	);
	end component;

	signal llc_clk : std_logic;
	signal llc_clk_180 : std_logic;

	signal sigin_wanted_r : std_logic;
	signal signals_in_r : std_logic_vector(31 downto 0);
	signal sigin_ready_r : std_logic;
	signal signals_out_r : std_logic_vector(31 downto 0);

	type state_type is (IDLE, HANDSHAKE);
	signal state : state_type;
begin
	pll : llc_pll
	port map
	(
		tm4_glbclk0,
		llc_clk,
		llc_clk_180
	);

	tm4_fir_coldfire <= '0';
	tm4_fir_lendian <= '0';
	tm4_fir_m8bit <= '0';
	tm4_fir_mcmode <= '1';
	tm4_fir_resetn <= reset_n;
	tm4_fir_bclk <= llc_clk_180;

	process (llc_clk)
	begin
		if rising_edge(llc_clk) then
			signals_in_r <= signals_in;
			sigin_ready_r <= sigin_ready;
		end if;
	end process;

	process (tm4_glbclk0)
	begin
		if rising_edge(tm4_glbclk0) then
			signals_out <= signals_out_r;
			sigin_wanted <= sigin_wanted_r;
		end if;
	end process;


	process (llc_clk, reset_n)
	begin
		if reset_n = '0' then
			state <= IDLE;
			sigin_wanted_r <= '1';
			tm4_fir_md <= (others => 'Z');
			tm4_fir_mcsn <= '1';
			tm4_fir_mwrn <= '1';
		elsif rising_edge(llc_clk) then
			case state is
			when IDLE =>
				if sigin_ready_r = '1' and tm4_fir_mcan = '1' then
					sigin_wanted_r <= '0';
					state <= HANDSHAKE;

					tm4_fir_ma <= signals_in_r(6 downto 0);
					tm4_fir_mcsn <= '0';

					if signals_in_r(31) = '1' then
						tm4_fir_mwrn <= '0';
						tm4_fir_md <= signals_in_r(23 downto 8);
					else
						tm4_fir_mwrn <= '1';
						tm4_fir_md <= (others => 'Z');
					end if;
				else
					sigin_wanted_r <= '1';
					tm4_fir_mwrn <= '1';
					tm4_fir_mcsn <= '1';
					tm4_fir_md <= (others => 'Z');
				end if;
			when HANDSHAKE =>
				if sigin_ready_r = '0' and tm4_fir_mcan = '0' then
					sigin_wanted_r <= '1';
					signals_out_r(15 downto 0) <= tm4_fir_md;
					signals_out_r(31 downto 16) <= (others => '0');
					tm4_fir_mwrn <= '1';
					tm4_fir_mcsn <= '1';
					state <= IDLE;
				end if;
			end case;
		end if;
	end process;
end rtl;
